<?php 
/**
 * Plugin Name: Pósturinn's Shipping with WooCommerce
 * Description: This plugin is an WooCommerce extension to extend shipping method with Pósturinn's - Iceland
 * Version: 1.1.0
 * Author: Pósturinn
 * Author URI: https://postur.is
 * Text Domain: postis
 * Domain Path: /languages
 * License: GPL2
 */


/* ======= Direct Access Not Allowed =========== */
if( ! defined('ABSPATH' ) ){ exit; }

/* ======= Define Constant =========== */
define('POSTIS_PATH', untrailingslashit(plugin_dir_path( __FILE__ )) );
define('POSTIS_URL', untrailingslashit(plugin_dir_url( __FILE__ )) );
define('POSTIS_PDF_DIR', 'posturinn' );

/* ======= Plugin Files Include =========== */
require_once(POSTIS_PATH.'/lib/fpdi/vendor/autoload.php');
use setasign\Fpdi\Tcpdf\Fpdi;

if( file_exists( dirname(__FILE__).'/inc/functions.php' )) include_once dirname(__FILE__).'/inc/functions.php';
if( file_exists( dirname(__FILE__).'/inc/hooks.php' )) include_once dirname(__FILE__).'/inc/hooks.php';
if( file_exists( dirname(__FILE__).'/inc/api.class.php' )) include_once dirname(__FILE__).'/inc/api.class.php';
if( file_exists( dirname(__FILE__).'/inc/admin.class.php' )) include_once dirname(__FILE__).'/inc/admin.class.php';
if( file_exists( dirname(__FILE__).'/inc/checkout-fields.class.php' )) include_once dirname(__FILE__).'/inc/checkout-fields.class.php';


class POSTIS_MAIN_CLASS {

    private static $ins = null;
    
    function __construct(){

        add_action ('init', array($this, 'load_textdomain'));
        
        add_action('woocommerce_shipping_init', array($this, 'load_shipping_method'));
        
        add_filter('woocommerce_shipping_methods', array($this, 'include_shipping_method'));
        
        add_action('woocommerce_review_order_after_shipping', 'postis_hooks_rendering_extra_shipping_services');
        
        add_action( 'wp_enqueue_scripts','postis_hooks_load_scripts', 10 );
        
        $gm_enable = postis_get_settings( 'googlemap_enable' );
        if ($gm_enable == 'yes') {
            add_action( 'woocommerce_after_checkout_form', 'postis_hooks_after_checkout_form', 20 );
        }
        
        $plugin_name = plugin_basename( __FILE__ );
        add_filter( "plugin_action_links_{$plugin_name}", array($this, 'plugin_setting_menu') );
    }
    
    
    /* 
    **============= Load Plugin TextDomain ================ 
    */
    function load_textdomain() {

    	$loadedok = load_plugin_textdomain('postis', false, basename( dirname( __FILE__ ) ) . '/languages');
    }
    
    
    /* 
    **============= Init Postis Shipping Method with all the settings ================ 
    */
    function load_shipping_method() {
        
        // Load main postis shipping class
        include POSTIS_PATH.'/inc/postis.class.php';
    }
    
    
    /* 
    **============= Include the shipping method to WC list ================ 
    */
    function include_shipping_method( $methods ) {
        
    	$methods[] = 'POSTIS_Shipping';
    	return $methods;
    }
    
    /* 
    **========== Create Setting Menu on plugin page =========== 
    */
    function plugin_setting_menu( $links ) {
        
        $setting_url    = admin_url( 'admin.php?page=wc-settings&tab=shipping&section=postis' );
        $title          = 'Posturinn Settings';
        $settings_link  = sprintf(__('<a href="%s">%s</a>','postis'), $setting_url, $title);
      	array_push( $links, $settings_link );
      	return $links;
    }
    
    
    /* 
    **============= Get class instance ================ 
    */
    public static function get_instance() {
        
        // create a new object if it doesn't exist.
        is_null(self::$ins) && self::$ins = new self;
        return self::$ins;
    }
}

add_action('plugins_loaded', 'postis_plugin_loaded');
function postis_plugin_loaded() {
    return POSTIS_MAIN_CLASS::get_instance();
}

// process PDF
function postis_process_pdf( $file_name, $args, $is_shipment_international ) {
	
	$dir_path = postis_files_setup_get_directory('pdf');
	$dir_path = "{$dir_path}{$file_name}";
	
	if( ! file_exists($dir_path) ) wp_die("No PDF Found");
	//postis_pa($args); exit;
	
	
	// initiate FPDI
	$pdf = new Fpdi();
	
    $pdf_width       = $args['pdf_width'];
    $pdf_height      = $args['pdf_height'];
    $pdf_rotate      = intval($args['pdf_rotate']);
    $pdf_orientation = $args['pdf_orientation'];
    $pdf_moveright    = intval($args['pdf_moveright']);
    $pdf_movedown     = intval($args['pdf_movedown']);
    $pdf_biggerbarcode= intval($args['pdf_biggerbarcode']);
    $numberOfItems    = intval($args['numberOfItems']);
    
    // Page size
    $pdf_size   = array($pdf_width, $pdf_height, 'Rotate' => $pdf_rotate);
    
    // set the source file
    $pageCount = $pdf->setSourceFile($dir_path);
    // var_dump($numberOfItems);
    for ($pageNo = 1; $pageNo <= $pageCount; $pageNo++) {
        $tplIdx = $pdf->importPage($pageNo);
    
        // add a page
        if ($is_shipment_international) {
            $pdf->AddPage();
        }else if ($numberOfItems > 1) {
            $pdf->AddPage();
        }else{
            $pdf->AddPage($pdf_orientation, $pdf_size);
        }
    
        $size   = $pdf->useTemplate($tplIdx, $pdf_moveright, $pdf_movedown, $pdf_biggerbarcode);
    }
    
    // set array for viewer preferences
    $preferences = array(
        'HideToolbar' => false,
        'HideMenubar' => false,
        'HideWindowUI' => false,
        'FitWindow' => true,
        'CenterWindow' => true,
        'DisplayDocTitle' => true,
        'NonFullScreenPageMode' => 'UseNone', // UseNone, UseOutlines, UseThumbs, UseOC
        'ViewArea' => 'CropBox', // CropBox, BleedBox, TrimBox, ArtBox
        'ViewClip' => 'CropBox', // CropBox, BleedBox, TrimBox, ArtBox
        //'PrintArea' => 'BleedBox', // CropBox, BleedBox, TrimBox, ArtBox
        //'PrintClip' => 'CropBox', // CropBox, BleedBox, TrimBox, ArtBox
        //'PrintScaling' => 'AppDefault', // None, AppDefault
        //'Duplex' => 'DuplexFlipLongEdge', // Simplex, DuplexFlipShortEdge, DuplexFlipLongEdge
        'PickTrayByPDFSize' => true,
        //'PrintPageRange' => array(1,1,2,3),
        'NumCopies' => 1
    );
    
    // Check the example n. 60 for advanced page settings
    
    // set pdf viewer preferences
    $pdf->setViewerPreferences($preferences);
	
	$pdf->Output();
}

function postis_merge_pdf_files($pdf_files, $pdf_path){
    $pdf_width         = postis_get_settings( 'pdf_width' );
    $pdf_height        = postis_get_settings( 'pdf_height' );
    $pdf_rotate        = postis_get_settings( 'pdf_rotate' );
    $pdf_orientation   = postis_get_settings( 'pdf_orientation' );
    $pdf_moveright     = postis_get_settings( 'pdf_moveright' );
    $pdf_movedown      = postis_get_settings( 'pdf_movedown' );
    $pdf_biggerbarcode = postis_get_settings( 'pdf_biggerbarcode' );
    $pdf_size   = array($pdf_width, $pdf_height, 'Rotate' => $pdf_rotate);
    
    $pdf = new FPDI();

    foreach ($pdf_files as $file) {
        
        if (!empty($file)) {
            
            $pageCount = $pdf->setSourceFile($file);
            
            for ($pageNo = 1; $pageNo <= $pageCount; $pageNo++) {
                
                // import a page
                $templateId = $pdf->importPage($pageNo);
                
                $pdf->AddPage($pdf_orientation,$pdf_size);
        
                $pdf->useTemplate($templateId, $pdf_moveright, $pdf_movedown, $pdf_biggerbarcode);
            }
        }
    }
    
    $pdf->Output($pdf_path,"F");
}
